package nl.amis.rest.resources;

import java.io.IOException;

import java.util.Collection;

import nl.amis.rest.controller.RestfulApplication;

import org.restlet.Context;
import org.restlet.data.Form;
import org.restlet.data.MediaType;
import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.data.Status;
import org.restlet.resource.DomRepresentation;
import org.restlet.resource.Representation;
import org.restlet.resource.Resource;
import org.restlet.resource.ResourceException;
import org.restlet.resource.StringRepresentation;
import org.restlet.resource.Variant;

import org.w3c.dom.Document;
import org.w3c.dom.Element;


public class ThingResource extends Resource {

    Thing thing;


    public ThingResource(Context context, Request request, Response response) {
        super(context, request, response);
        // Declare the kind of representations supported by this resource.
        getVariants().add(new Variant(MediaType.TEXT_XML));
        // get the thingName parameter from the URL /things/{thingName}
        String thingName =
            (String)getRequest().getAttributes().get("thingName");
        // Allow modifications of this resource via POST requests.
        setModifiable(true);

        // Get the things directly from the "persistence layer".
        thing = getTheThing(thingName);

    }

    /**
     * Returns a listing of all registered items.
     */
    @Override
    public Representation represent(Variant variant) throws ResourceException {
        // Generate the right representation according to its media type.
        if (MediaType.TEXT_XML.equals(variant.getMediaType())) {
            try {
                DomRepresentation representation =
                    new DomRepresentation(MediaType.TEXT_XML);
                Document d = representation.getDocument();
                Element eltThing = d.createElement("thing");
                d.appendChild(eltThing);

                Element eltName = d.createElement("name");
                eltName.appendChild(d.createTextNode(thing.getName()));
                eltThing.appendChild(eltName);

                Element eltDescription = d.createElement("description");
                eltDescription.appendChild(d.createTextNode(thing.getSomeOtherProperty()));
                eltThing.appendChild(eltDescription);

                Element eltAndThisToo = d.createElement("andThisToo");
                eltAndThisToo.appendChild(d.createTextNode(thing.getAndThisToo()));
                eltThing.appendChild(eltAndThisToo);

                Element eltImportantValue = d.createElement("importantValue");
                eltImportantValue.appendChild(d.createTextNode(Integer.toString(thing.getImportantValue())));
                eltThing.appendChild(eltImportantValue);

                d.normalizeDocument();

                // Returns the XML representation of this document.
                return representation;
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        return null;
    }

    /**
     * Handle POST requests: create a new thing.
     */
    @Override
    public void acceptRepresentation(Representation entity) throws ResourceException {
        // Parse the given representation and retrieve pairs of
        // "name=value" tokens.
        Form form = new Form(entity);
        Representation rep = null;
        if (thing!=null) { // it exists! 
            String description = form.getFirstValue("description");
            thing.setSomeOtherProperty(description);
            getResponse().setStatus(Status.SUCCESS_CREATED);
            rep=
                new StringRepresentation("Thing updated", MediaType.TEXT_PLAIN);
            // Indicates where is located the new resource.
            rep.setIdentifier(getRequest().getResourceRef().getIdentifier() + "/" +
                              thing.getName());
        }
        else {
            getResponse().setStatus(Status.CLIENT_ERROR_NOT_FOUND);            
            rep=
                new StringRepresentation("Thing could not be found", MediaType.TEXT_PLAIN);
        }
        // Set the response's status and entity
        getResponse().setEntity(rep);
    }


    private Thing getTheThing(String thingName) {
        Collection<Thing> things =
            ((RestfulApplication)getApplication()).getThings();
        for (Thing thing : things) {
            if (thingName.equalsIgnoreCase(thing.getName()))
                return thing;
        }
        return null;
    }
}
