package nl.amis.rest.resources;

import java.io.IOException;

import java.util.Collection;

import nl.amis.rest.controller.RestfulApplication;

import org.restlet.Context;
import org.restlet.data.Form;
import org.restlet.data.MediaType;
import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.data.Status;
import org.restlet.resource.DomRepresentation;
import org.restlet.resource.Representation;
import org.restlet.resource.Resource;
import org.restlet.resource.ResourceException;
import org.restlet.resource.StringRepresentation;
import org.restlet.resource.Variant;

import org.w3c.dom.Document;
import org.w3c.dom.Element;


public class ThingsResource extends Resource {

    Collection<Thing> things;


    public ThingsResource(Context context, Request request,
                          Response response) {
        super(context, request, response);
        // Declare the kind of representations supported by this resource.
        getVariants().add(new Variant(MediaType.TEXT_XML));
        // Allow modifications of this resource via POST requests.
        setModifiable(true);

        // Get the things directly from the "persistence layer".
        things = getThings();

    }

    /**
     * Returns a listing of all registered items.
     */
    @Override
    public Representation represent(Variant variant) throws ResourceException {
        // Generate the right representation according to its media type.
        if (MediaType.TEXT_XML.equals(variant.getMediaType())) {
            try {
                DomRepresentation representation =
                    new DomRepresentation(MediaType.TEXT_XML);
                // Generate a DOM document representing the list of
                // items.
                Document d = representation.getDocument();
                Element r = d.createElement("things");
                d.appendChild(r);
                for (Thing thing : things) {
                    Element eltThing = d.createElement("thing");

                    Element eltName = d.createElement("name");
                    eltName.appendChild(d.createTextNode(thing.getName()));
                    eltThing.appendChild(eltName);

                    Element eltDescription = d.createElement("description");
                    eltDescription.appendChild(d.createTextNode(thing.getSomeOtherProperty()));
                    eltThing.appendChild(eltDescription);

                    Element eltAndThisToo = d.createElement("andThisToo");
                    eltAndThisToo.appendChild(d.createTextNode(thing.getAndThisToo()));
                    eltThing.appendChild(eltAndThisToo);

                    Element eltImportantValue =
                        d.createElement("importantValue");
                    eltImportantValue.appendChild(d.createTextNode(Integer.toString(thing.getImportantValue())));
                    eltThing.appendChild(eltImportantValue);

                    r.appendChild(eltThing);
                }
                d.normalizeDocument();

                // Returns the XML representation of this document.
                return representation;
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        return null;
    }


    /**
     * Handle POST requests: create a new thing.
     */
    @Override
    public void acceptRepresentation(Representation entity) throws ResourceException {
        // Parse the given representation and retrieve pairs of
        // "name=value" tokens.
        Form form = new Form(entity);
        String name = form.getFirstValue("name");
        String description = form.getFirstValue("description");

        // Register the new item
        getThings().add(new Thing(name, description, "", 0, 0));

        // Set the response's status and entity
        getResponse().setStatus(Status.SUCCESS_CREATED);
        Representation rep =
            new StringRepresentation("Thing created", MediaType.TEXT_PLAIN);
        // Indicates where is located the new resource.
        rep.setIdentifier(getRequest().getResourceRef().getIdentifier() + "/" +
                          name);
        getResponse().setEntity(rep);
    }


    /**
     * Returns the collection of things managed by this application.
     *
     * @return the collection of things managed by this application.
     */
    protected Collection<Thing> getThings() {
        return ((RestfulApplication)getApplication()).getThings();
    }


}
